package com.jh.boot.attachment;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.springframework.web.multipart.MultipartFile;

public interface AttachmentService {

    Attachment saveAttachment(Attachment attachment, InputStream content);

    Attachment saveAttachment(Attachment attachment);

    List<Attachment> fetchAttachments(Long unitId, String objectType, Long objectId);

    Attachment fetchAttachment(Long attachmentId);

    InputStream fetchAttachmentBody(Attachment attachment);

    void deleteAttachment(Long attachmentId);

    
    // default methods
    
    public default Attachment addAttachment(String objectType, Long objectId, MultipartFile file) {
        return addAttachment(null, objectType, objectId, file);
    }

    public default Attachment addAttachment(Long unitId, String objectType, Long objectId, MultipartFile file) {
        Attachment a = new Attachment();
        a.setFilename(file.getOriginalFilename());
        a.setSize(file.getSize());
        a.setObjectId(objectId);
        a.setObjectType(objectType);
        try {
            saveAttachment(a, file.getInputStream());
        } catch (Exception e) {
            throw new RuntimeException("Cannot upload attachment", e);
        }

        return a;
    }

    public default Attachment syncAttachment(String objectType, Long objectId, Attachment currentAttachment, Attachment newAttachment, MultipartFile file) {
        return syncAttachment(null, objectType, objectId, currentAttachment, newAttachment, file);
    }

    public default Attachment syncAttachment(Long unitId, String objectType, Long objectId, Attachment currentAttachment, Attachment newAttachment, MultipartFile file) {
        Attachment a = null;
        if (currentAttachment != null && newAttachment == null) {
            //delete current attachment
            deleteAttachment(currentAttachment.getId());
        }
        if (file != null) {
            a = addAttachment(unitId, objectType, objectId, file);
        } else {
            a = currentAttachment;
        }
        return a;
    }

    
    public default <C extends Collection<Attachment>> C syncAttachments(String objectType, Long objectId, C currentAttachments, C newAttachments, MultipartFile[] files, Class<C> clazz) {
        return syncAttachments(null, objectType, objectId, currentAttachments, newAttachments, files, clazz);
    }
    

    public default <C extends Collection<Attachment>> C syncAttachments(Long unitId, String objectType, Long objectId, C currentAttachments, C newAttachments, MultipartFile[] files, Class<C> clazz) {
        //decide whether Set or List
        if (currentAttachments == null) {
            if (Set.class.isAssignableFrom(clazz)) {
            	currentAttachments = (C) new HashSet<Attachment>();
            } else if (List.class.isAssignableFrom(clazz)) {
            	currentAttachments = (C) new ArrayList<Attachment>();
            } else {
                throw new IllegalArgumentException("Invalid collection type");
            }
        }
        
        if (newAttachments == null || newAttachments.isEmpty()) {
        	currentAttachments.clear();
        } else {
        	Iterator<Attachment> it = currentAttachments.iterator();
        	while (it.hasNext()) {
        		Attachment a = it.next();
        		if (!newAttachments.contains(a)) {
        			it.remove();
        		}
        	}
        }
        
        //TODO add attachments?
        
        if (files != null && files.length > 0) {
        	for (MultipartFile f : files) {
        		currentAttachments.add(addAttachment(unitId, objectType, objectId, f));
        	}
        }
        
        
        return currentAttachments;
        
    }
    
    
}
