package com.jh.boot.jpa;

import java.util.*;

//TODO move to jh-boot
public abstract class CollectionSync<R extends AbstractIdEntity, T extends AbstractIdEntity> {

    private AbstractHibernateRepository repo;

    private boolean deleteOrphans;

    protected CollectionSync(AbstractHibernateRepository repo) {
        this(repo, false);
    }

    protected CollectionSync(AbstractHibernateRepository repo, boolean deleteOrphans) {
        this.repo = repo;
        this.deleteOrphans = deleteOrphans;
    }


    public <C extends Collection<T>> C synchronize(R root, C existing, Collection<T> fresh, Class<C> clazz) {
        //decide whether Set or List
        if (existing == null) {
            if (Set.class.isAssignableFrom(clazz)) {
                existing = (C) new HashSet<T>();
            } else if (List.class.isAssignableFrom(clazz)) {
                existing = (C) new ArrayList<T>();
            } else {
                throw new IllegalArgumentException("Invalid collection type");
            }
        }

        List<T> toBeAdded = new ArrayList<>();
        Map<Long, T> map = new HashMap();
        if (fresh != null) {
            for (T t : fresh) {
                if (t.getId() == null) {
                    toBeAdded.add(t);
                } else {
                    map.put(t.getId(), t);
                }
            }
        }

        Iterator<T> it = existing.iterator();
        while (it.hasNext()) {
            T e = it.next();
            T f = map.get(e.getId());
            if (f == null) {
                it.remove();
                doRemove(e);
                continue;
            }

            doMerge(e, f);
        }

        for (T t : toBeAdded) {
            setRootToEntity(root, t);
            doPersistNew(t);
            existing.add(t);
        }

        return existing;
    }


    protected void doRemove(T entity) {
        if (deleteOrphans) {
            repo.delete(entity);
        }
    }

    protected void doPersistNew(T entity) {
        repo.save(entity);
    }

    protected abstract void setRootToEntity(R root, T entity);

    protected abstract void doMerge(T existing, T fresh);

    protected boolean isDeleteOrphans() {
        return deleteOrphans;
    }

    protected AbstractHibernateRepository getRepo() {
        return repo;
    }
}
